import {
	ActionType,
	DRP,
	type ResolveConflictsType,
	SemanticsType,
	type Vertex,
} from "@sfroment/object";

type Insert = {
	peerId: string;
	content: string;
	color: string;
};

export class ExampleDRPObject implements DRP {
	semanticsType = SemanticsType.pair;

	private _inserts: Insert[] = [];
	private _peerId: string;
	constructor(peerId: string) {
		this._inserts = [];
		this._peerId = peerId;
	}

	insert(peerId: string, index: number, content: string) {
		if (content.length === 0) return;

		this._inserts.splice(index, 0, {
			peerId,
			content,
			color: "red",
		});
	}

	delete(index: number, length: number) {
		this._inserts.splice(index, length);
	}

	// insert -> first
	// insert -> delete
	// insert -> insert
	// insert the insert hisght peerID 1st
	// delete the delete highest peerID 1st
	resolveConflicts(vertices: Vertex[]): ResolveConflictsType {
		const opInsert = "insert";
		//const opDelete = "delete";

		if (vertices.length <= 1 || vertices[0].hash === vertices[1].hash) {
			return { action: ActionType.Nop };
		}

		const leftOp = vertices[0].operation?.opType;
		const rightOp = vertices[1].operation?.opType;
		const leftPeerId = vertices[0].peerId;
		const rightPeerId = vertices[1].peerId;
		const leftContent = vertices[0].operation?.value[2];
		const rightContent = vertices[1].operation?.value[2];

		if (leftOp === rightOp) {
			if (leftPeerId > rightPeerId) {
				return { action: ActionType.Swap };
			}

			if (leftPeerId === rightPeerId && leftContent > rightContent) {
				return { action: ActionType.Swap };
			}
		}

		if (leftOp !== rightOp) {
			if (rightOp === opInsert) {
				return { action: ActionType.Swap };
			}
		}

		return { action: ActionType.Nop };
	}

	query_insert(): Insert[] {
		return this._inserts;
	}

	query_text(): string {
		return this._inserts.map((insert) => insert.content).join("");
	}
}

function whoIsWho(peerId: string) {
	if (peerId === "12D3KooWHJZQSpNVSHXuMvEt76j3zmtXVfDexYmhNm5fWSo99fxQ") {
		return "peer2";
	}
	return "peer1";
}
